@@ -6,7 +6,7 @@ import copy
 
 from tsfpga.constraint import Constraint
 from tsfpga.hdl_file import HdlFile
-from tsfpga.registers import from_json
+from tsfpga.register_list import from_json
 from tsfpga.system_utils import load_python_module
 
 
@@ -18,17 +18,16 @@ class BaseModule:
     different catalog structure.
     """
 
-    def __init__(self, path, library_name_has_lib_suffix=False, default_registers=None):
+    def __init__(self, path, library_name, default_registers=None):
         """
         Args:
             path (`pathlib.Path`): Path to the module folder.
-            library_name_has_lib_suffix (bool): If set, the library name will be
-                <module name>_lib, otherwise it is just <module name>.
-            default_registers (dict[str, Register]): Default registers.
+            library_name (str): VHDL library name.
+            default_registers (list(Register)): Default registers.
         """
         self.path = path
         self.name = path.name
-        self.library_name = self._get_library_name(self.name, library_name_has_lib_suffix)
+        self.library_name = library_name
 
         # Note: Likely mutable object, need to create deep copy before using.
         self._default_registers = default_registers
@@ -55,19 +54,38 @@ class BaseModule:
     @property
     def registers(self):
         """
-        Get the :class:`.Registers` for this module.
+        :class:`.RegisterList`: Get the registers for this module.
         """
         if self._registers is not None:
             # Only create object once
             return self._registers
 
-        json_file = self.path / (self.name + "_regs.json")
+        json_file = self.path / (f"regs_{self.name}.json")
         if json_file.exists():
             self._registers = from_json(self.name, json_file, copy.deepcopy(self._default_registers))
+            self.registers_hook()
             return self._registers
 
+        deprecated_json_file = self.path / (self.name + "_regs.json")
+        if deprecated_json_file.exists():
+            message = f"DEPRECATED: Using deprecated json file name: {deprecated_json_file}"
+            message += f"\nDEPRECATED: Rename to {json_file}"
+            raise ValueError(message)
+
         return None
 
+    def registers_hook(self):
+        """
+        This function will be called directly after creating this module's registers from
+        the JSON definition file.
+
+        This is a good place if you want to add or modify some registers from Python.
+        Override this method and implement the desired behavior in a child class.
+
+        .. Note::
+            This default method does nothing and can be safely overridden.
+        """
+
     def create_regs_vhdl_package(self):
         """
         Create a VHDL package in this module with register definitions.
@@ -78,7 +96,7 @@ class BaseModule:
     def get_synthesis_files(self):
         """
         Return:
-            List of files (:class:`.HdlFile` objects) that should be included in a synthesis project.
+            list(:class:`.HdlFile`): Files that should be included in a synthesis project.
         """
         self.create_regs_vhdl_package()
 
@@ -105,8 +123,7 @@ class BaseModule:
                     other modules.
 
         Return:
-            List of files (:class:`.HdlFile` objects) that should be included in a
-            simulation project.
+            list(:class:`.HdlFile`): Files that should be included in a simulation project.
         """
         self.create_regs_vhdl_package()
 
@@ -120,20 +137,13 @@ class BaseModule:
 
         return self.get_synthesis_files() + self._get_hdl_file_list(test_folders)
 
-    @staticmethod
-    def _get_library_name(module_name, library_name_has_lib_suffix):
-        """
-        By praxis VHDL libraries should be named e.g. ieee and not ieee_lib.
-        However using <module_name>_lib is very common.
-        """
-        if library_name_has_lib_suffix:
-            return module_name + "_lib"
-        return module_name
-
     def setup_simulations(self, vunit_proj, **kwargs):
         """
         Setup local configuration of this module's test benches.
-        Should be overridden by modules that have any test benches that operate via generics.
+
+        .. Note::
+            This default method does nothing. Should be overridden by modules that have
+            any test benches that operate via generics.
 
         Args:
             vunit_proj: The VUnit project that is used to run simulation.
@@ -144,7 +154,8 @@ class BaseModule:
 
     def get_ip_core_files(self):
         """
-        Get a list of TCL files that set up the IP cores from this module.
+        Return:
+            list(`pathlib.Path`): TCL files that set up the IP cores from this module.
         """
         folders = [
             self.path / "ip_cores",
@@ -154,7 +165,9 @@ class BaseModule:
 
     def get_scoped_constraints(self):
         """
-        Get a list of constraints that will be applied to a certain entity within the module.
+        Return:
+            list(:class:`.Constraint`): Constraints that will be applied to a certain
+            entity within the module.
         """
         scoped_constraints_folders = [
             self.path / "scoped_constraints",
@@ -175,8 +188,20 @@ class BaseModule:
                 constraints.append(constraint)
         return constraints
 
+    def get_build_projects(self):  # pylint: disable=no-self-use
+        """
+        Get FPGA build projects defined by this module.
+
+        .. Note::
+            Should be overridden by modules that set up build projects.
+
+        Return:
+            list(:class:`.VivadoProject`): FPGA build projects.
+        """
+        return []
+
     def __str__(self):
-        return self.name + ": " + self.path
+        return f"{self.name}:{self.path}"
 
 
 def iterate_module_folders(modules_folders):
@@ -187,13 +212,12 @@ def iterate_module_folders(modules_folders):
 
 
 def get_module_object(path, name, library_name_has_lib_suffix, default_registers):
-    module_file = path / ("module_" + name + ".py")
+    module_file = path / f"module_{name}.py"
+    library_name = f"{name}_lib" if library_name_has_lib_suffix else name
 
     if module_file.exists():
-        return load_python_module(module_file).Module(path,
-                                                      library_name_has_lib_suffix,
-                                                      default_registers)
-    return BaseModule(path, library_name_has_lib_suffix, default_registers)
+        return load_python_module(module_file).Module(path, library_name, default_registers)
+    return BaseModule(path, library_name, default_registers)
 
 
 def get_modules(modules_folders,
@@ -208,8 +232,9 @@ def get_modules(modules_folders,
         modules_folders (list(`pathlib.Path`)): A list of paths where your modules are located.
         names_include (list(str)): If specified, only modules with these names will be included.
         names_avoid (list(str)): If specified, modules with these names will be discarded.
-        library_name_has_lib_suffix (bool): See :class:`BaseModule`.
-        default_registers: See :class:`BaseModule`.
+        library_name_has_lib_suffix (bool): If set, the library name will be
+            ``<module name>_lib``, otherwise it is just ``<module name>``.
+        default_registers (list(Register)): Default registers.
 
     Return:
         List of module objects (:class:`BaseModule` or child classes thereof) created from
@@ -226,4 +251,10 @@ def get_modules(modules_folders,
                                              library_name_has_lib_suffix,
                                              default_registers))
 
+            deprecated_project_file = module_folder / f"project_{module_name}.py"
+            if deprecated_project_file.exists():
+                message = f"DEPRECATED: Using deprecated project file: {deprecated_project_file}"
+                message += f"\nDEPRECATED: Instead use module_{module_name}.py"
+                raise ValueError(message)
+
     return modules
