@@ -2,6 +2,8 @@
 # Copyright (c) Lukas Vik. All rights reserved.
 # ------------------------------------------------------------------------------
 
+from pathlib import Path
+
 import pytest
 
 from tsfpga.constraint import Constraint
@@ -9,13 +11,13 @@ from tsfpga.hdl_file import HdlFile
 
 
 def test_constraint():
-    constraint = Constraint("dummy.tcl")
+    constraint = Constraint(Path("dummy.tcl"))
     constraint.validate_scoped_entity([])
 
     assert constraint.ref is None
     assert constraint.used_in == "all"
 
-    constraint = Constraint("dummy.tcl", used_in="impl")
+    constraint = Constraint(Path("dummy.tcl"), used_in="impl")
     assert constraint.used_in == "impl"
 
 
@@ -32,3 +34,7 @@ def test_matching_entity_not_existing_should_raise_exception(tmp_path):
     with pytest.raises(FileNotFoundError) as exception_info:
         constraint.validate_scoped_entity([])
     assert str(exception_info.value).startswith("Could not find a matching entity file")
+
+
+def test_can_cast_to_string_without_error():
+    str(Constraint(Path("dummy.tcl"), used_in="impl"))
